@@ -75,12 +75,14 @@ parse_jvm_options() {
 
 setup_java() {
   # set the path to java into JAVACMD which will be picked up by JRuby to launch itself
-  if [ -x "$JAVA_HOME/bin/java" ]; then
-    JAVACMD="$JAVA_HOME/bin/java"
-  else
-    set +e
-    JAVACMD=`command -v java`
-    set -e
+  if [ -z "$JAVACMD" ]; then
+    if [ -x "$JAVA_HOME/bin/java" ]; then
+      JAVACMD="$JAVA_HOME/bin/java"
+    else
+      set +e
+      JAVACMD=`command -v java`
+      set -e
+    fi
   fi
 
   if [ ! -x "$JAVACMD" ]; then
@@ -138,7 +140,7 @@ setup_vendored_jruby() {
   fi
 
   if [ -z "$LS_GEM_HOME" ] ; then
-    export GEM_HOME="${LOGSTASH_HOME}/vendor/bundle/jruby/2.3.0"
+    export GEM_HOME="${LOGSTASH_HOME}/vendor/bundle/jruby/2.5.0"
   else
     export GEM_HOME=${LS_GEM_HOME}
   fi
