@@ -1,4 +1,20 @@
-# encoding: utf-8
+# Licensed to Elasticsearch B.V. under one or more contributor
+# license agreements. See the NOTICE file distributed with
+# this work for additional information regarding copyright
+# ownership. Elasticsearch B.V. licenses this file to you under
+# the Apache License, Version 2.0 (the "License"); you may
+# not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#  http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing,
+# software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+# KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations
+# under the License.
+
 Thread.abort_on_exception = true
 Encoding.default_external = Encoding::UTF_8
 $DEBUGLIST = (ENV["DEBUG"] || "").split(",")
@@ -13,6 +29,10 @@ class LogStash::DependencyReport < Clamp::Command
   option [ "--csv" ], "OUTPUT_PATH", "The path to write the dependency report in csv format.",
     :required => true, :attribute_name => :output_path
 
+  OTHER_DEPENDENCIES = [
+    ["jruby", "", "http://jruby.org", "EPL-2.0"]
+  ]
+
   def execute
     require "csv"
 
@@ -24,6 +44,8 @@ class LogStash::DependencyReport < Clamp::Command
       gems.each { |d| csv << d }
       puts "Finding gem embedded java/jar dependencies"
       jars.each { |d| csv << d }
+      puts "Adding non-gem non-jar dependencies (such as jruby distribution)"
+      OTHER_DEPENDENCIES.each { |d| csv << d }
     end
     puts "Wrote temporary ruby deps CSV to #{ruby_output_path}"
 
